package cn.com.qmth.export;

import java.io.File;
import java.io.IOException;
import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;

import oracle.jdbc.driver.OracleDriver;

public class ExportPaperByAllQuestions {
	private static Logger logger = LogManager.getLogger(Export.class);
	private static String courseCode="AAAAA";
	private static String excelDir;
	private static String quesDir;
	private static Map<String, String> quesTypes = new HashMap<String, String>();
	private static Map<String, String> mutiQuesTypes = new HashMap<String, String>();
	static {
		quesTypes.put("11", "单选题");
		quesTypes.put("13", "填空题");
		quesTypes.put("20", "判断题");
		quesTypes.put("69", "名词解释");
		quesTypes.put("12", "多选题");
		quesTypes.put("60", "问答题");
	}
	static {
		mutiQuesTypes.put("42", "复合题");
		mutiQuesTypes.put("14", "选词填空");
		mutiQuesTypes.put("15", "完形填空");
	}

	public static void dispose() throws InvalidFormatException, IOException, SQLException {
		File directory = new File("");
		excelDir = directory.getAbsolutePath() + "\\excelDir\\";
		quesDir = excelDir + "quesDir\\";
		File excelFolder = new File(excelDir);
		if (excelFolder.exists()) {
			FileUtil.clearDirectory(excelDir);
		} else {
			excelFolder.mkdir();
		}
		File quesFolder = new File(quesDir);
		quesFolder.mkdir();

		Connection connect = null;

		try {
			Driver driver = new OracleDriver();
			DriverManager.deregisterDriver(driver);

			Properties pro = new Properties();
			pro.put("user", "qspad");
			pro.put("password", "qspad1");
			connect = driver.connect("jdbc:oracle:thin:@202.114.196.115:1521:qspad", pro);
			for(String k:mutiQuesTypes.keySet()) {
				exportFuhePaper(connect, k);
			}
			for(String k:quesTypes.keySet()) {
				exportPaper(connect, k);
			}
		} finally {
			if (connect != null) {
				connect.close();
			}
		}
	}


	private static void exportFuhePaper(Connection connect,String questionType) throws SQLException, IOException {

		List<TestPaper> tps = new ArrayList<TestPaper>();
		TestPaper tp = new TestPaper();
		tp.setId(questionType);
		tp.setName(mutiQuesTypes.get(questionType)+"试卷(202)");
		tp.setTestPaperContentId(questionType);
		tp.setScore(100.0);
		tps.add(tp);
		List<SheetData> sheets = new ArrayList<SheetData>();
		List<String> paperheader = new ArrayList<String>();
		paperheader.addAll(Arrays.asList(new String[] { "试卷ID", "试卷名称", "试卷内容ID", "试卷满分" }));
		fillPaperData(sheets, tps, paperheader);
		int flag = 1;
		exportFuheQuestion(connect, questionType, sheets, flag);
		ExportUtils.exportExcel(excelDir, courseCode, sheets);
	}

	private static void exportPaper(Connection connect,String questionType) throws SQLException, IOException {

		List<TestPaper> tps = new ArrayList<TestPaper>();
		TestPaper tp = new TestPaper();
		tp.setId(questionType);
		tp.setName(quesTypes.get(questionType)+"试卷(202)");
		tp.setTestPaperContentId(questionType);
		tp.setScore(100.0);
		tps.add(tp);
		List<SheetData> sheets = new ArrayList<SheetData>();
		List<String> paperheader = new ArrayList<String>();
		paperheader.addAll(Arrays.asList(new String[] { "试卷ID", "试卷名称", "试卷内容ID", "试卷满分" }));
		fillPaperData(sheets, tps, paperheader);
		int flag = 1;
		exportQuestion(connect, questionType, sheets, flag);
		ExportUtils.exportExcel(excelDir, courseCode, sheets);
	}


	private static void exportFuheQuestion(Connection connect, String questionType, List<SheetData> sheets, int flag)
			throws SQLException, IOException {
		PreparedStatement preState = null;
		ResultSet resultSet = null;

		try {
			String sql = "with tem as (select h.question_content_id qid from question_info h  where h.question_type_enum ='"
					+ questionType + "' and h.batch_type='1' )"
					+ "SELECT q.ID id,'0' dNumber,'"+mutiQuesTypes.get(questionType)+"' dName,q.sequence number1,'1' score,q.PARENT_ID parent_id,q.QUESTION_TYPE_ENUM type,q.JSON "
					+ "					FROM QUESTION_CONTENT q,tem"
					+ "          where q.id in tem.qid or q.parent_id in tem.qid"
					+ "          order by q.create_time,q.parent_id,q.sequence";
			preState = connect.prepareStatement(sql);

			resultSet = preState.executeQuery();
			List<TestQuestion> tqs = new ArrayList<TestQuestion>();
			while (resultSet.next()) {
				TestQuestion tq = new TestQuestion();
				tq.setId(resultSet.getString("id"));
				tq.setdNumber(resultSet.getInt("dnumber"));
				tq.setdName(resultSet.getString("dname"));
				tq.setNumber(resultSet.getInt("number1"));
				tq.setScore(resultSet.getDouble("score"));
				tq.setParentId(resultSet.getString("parent_id"));
				tq.setType(resultSet.getInt("type"));
				tq.setJson(resultSet.getString("json"));
				tqs.add(tq);
			}
			List<String> header = new ArrayList<String>();
			header.addAll(Arrays.asList(new String[] { "试题ID", "大题号", "大题名称", "小题号", "题分", "题父ID", "题类型" }));
			fillQuestionData(sheets, tqs, header, flag);
		} finally {
			if (resultSet != null) {
				resultSet.close();
			}
			if (preState != null) {
				preState.close();
			}
		}
	}


	public static void fillPaperData(List<SheetData> sheets, List<TestPaper> list, List<String> header) {
		SheetData sheet = new SheetData();
		sheet.setHeader(header);
		sheet.setName("课程试卷信息");
		List<Object[]> data = new ArrayList<Object[]>();
		for (TestPaper b : list) {
			Object[] ob = new Object[header.size()];
			ob[0] = b.getId();
			ob[1] = b.getName();
			ob[2] = b.getTestPaperContentId();
			ob[3] = b.getScore();
			data.add(ob);
		}
		sheet.setData(data);
		sheets.add(sheet);
	}

	private static void exportQuestion(Connection connect, String questionType, List<SheetData> sheets, int flag)
			throws SQLException, IOException {
		PreparedStatement preState = null;
		ResultSet resultSet = null;

		try {
			String sql = "SELECT q.ID id,'0' dNumber,'"+mutiQuesTypes.get(questionType)+"' dName,q.sequence number1,'1' score,q.PARENT_ID parent_id,q.QUESTION_TYPE_ENUM type,q.JSON" + 
					"					FROM QUESTION_CONTENT q,question_info h" + 
					"          where h.question_type_enum ='11' and h.batch_type='1'" + 
					"          and q.id =h.question_content_id and q.parent_id ='00000000000000000000000000000000'" + 
					"          order by q.create_time";
			preState = connect.prepareStatement(sql);

			resultSet = preState.executeQuery();
			List<TestQuestion> tqs = new ArrayList<TestQuestion>();
			while (resultSet.next()) {
				TestQuestion tq = new TestQuestion();
				tq.setId(resultSet.getString("id"));
				tq.setdNumber(resultSet.getInt("dnumber"));
				tq.setdName(resultSet.getString("dname"));
				tq.setNumber(resultSet.getInt("number1"));
				tq.setScore(resultSet.getDouble("score"));
				tq.setParentId(resultSet.getString("parent_id"));
				tq.setType(resultSet.getInt("type"));
				tq.setJson(resultSet.getString("json"));
				tqs.add(tq);
			}
			List<String> header = new ArrayList<String>();
			header.addAll(Arrays.asList(new String[] { "试题ID", "大题号", "大题名称", "小题号", "题分", "题父ID", "题类型" }));
			fillQuestionData(sheets, tqs, header, flag);
		} finally {
			if (resultSet != null) {
				resultSet.close();
			}
			if (preState != null) {
				preState.close();
			}
		}
	}

	public static void fillQuestionData(List<SheetData> sheets, List<TestQuestion> list, List<String> header, int flag)
			throws IOException {
		SheetData sheet = new SheetData();
		sheet.setHeader(header);
		sheet.setName("试卷" + flag);
		List<Object[]> data = new ArrayList<Object[]>();
		for (TestQuestion b : list) {
			Object[] ob = new Object[header.size()];
			ob[0] = b.getId();
			ob[1] = b.getdNumber();
			ob[2] = b.getdName();
			ob[3] = b.getNumber();
			ob[4] = b.getScore();
			ob[5] = b.getParentId();
			ob[6] = b.getType();
			File txtFile = new File(quesDir + b.getId() + ".txt");
			if (!txtFile.exists()) {
				FileUtil.writeTxt(quesDir, b.getId(), b.getJson());
			}
			data.add(ob);
		}
		sheet.setData(data);
		sheets.add(sheet);
	}

}
